/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.usm.json;

import static com.openexchange.usm.connector.commands.CommandConstants.SESSIONID;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import javax.servlet.http.HttpSession;
import com.openexchange.usm.json.response.ResponseStatusCode;

/**
 * {@link HttpSessionUtil} Helper class so that USM can work in standalone mode with a servlet container that doesn't support attributes of
 * HTTP sessions.
 * 
 * @author <a href="mailto:afe@microdoc.de">Alexander Feess</a>
 */
public class HttpSessionUtil {

    private static final boolean _DISABLE_STANDALONE_SUPPORT = false;

    private static final Map<String, USMSessionCredentials> _SESSIONS = new ConcurrentHashMap<String, USMSessionCredentials>(16, 0.75f, 1);

    private HttpSessionUtil() {
    }

    public static void storeSession(HttpSession httpSession, String randomHttpSessionID, USMSessionCredentials usmSessionID) {
        try {
            httpSession.setAttribute(randomHttpSessionID, usmSessionID); // save the USM session in the HttpSession
        } catch (UnsupportedOperationException e) {
            if (_DISABLE_STANDALONE_SUPPORT)
                throw e;
            _SESSIONS.put(randomHttpSessionID, usmSessionID);
        }
    }

    public static USMSessionCredentials getSession(HttpSession httpSession, String sessionID) throws USMJSONAPIException {
        USMSessionCredentials creds = findSession(httpSession, sessionID);
        if (creds != null)
            return creds;
        throw new USMJSONAPIException(
            ConnectorBundleErrorCodes.COMMAND_UNKNOWN_SESSION,
            ResponseStatusCode.UNKNOWN_SESSION,
            "Unknown " + SESSIONID);
    }

    private static USMSessionCredentials findSession(HttpSession httpSession, String sessionID) {
        try {
            return (USMSessionCredentials) httpSession.getAttribute(sessionID);
        } catch (UnsupportedOperationException e) {
            if (_DISABLE_STANDALONE_SUPPORT)
                throw e;
            return _SESSIONS.get(sessionID);
        }
    }

    public static void removeSession(HttpSession httpSession, String sessionID) {
        try {
            httpSession.removeAttribute(sessionID);
        } catch (UnsupportedOperationException e) {
            if (_DISABLE_STANDALONE_SUPPORT)
                throw e;
            _SESSIONS.remove(sessionID);
        }
    }
}
